require_relative 'core/application'
require_relative 'core/version'

require 'mustermann'
require 'rack'
require 'tilt'

RACK_ENV = ENV['RACK_ENV'] ||= 'development' unless defined?(RACK_ENV)

module Fenix
  class << self
    ####
    # @example
    #   Fenix.root('Application', 'Views')
    #   # => '/Home/Fenix/Application/Views'
    #
    #   Fenix.root
    #   # => '/Home/Fenix'
    #
    # @return [String]
    # @since 1.0.0
    # @api public
    def root(*paths)
      File.join(FENIX_ROOT, *paths)
    end

    ####
    # Returns the current environmental state
    ####
    #
    # @example
    #   Fenix.environment
    #   # => :production
    #
    #   Fenix.environment
    #   # => :test
    #
    # @return [Symbol]
    # @since 1.0.0
    # @api public
    def environment
      @_environment ||= RACK_ENV.to_sym.downcase
    end
    alias_method :env, :environment

    ####
    # @example
    #   Fenix.before_load do
    #     # ...
    #   end
    #
    # @return [Array<Proc>]
    # @since 1.0.0
    # @api public
    def before_load(&block)
      @_before_load ||= []
      @_before_load << block if block_given?
      @_before_load
    end

    ####
    # @example
    #   Fenix.after_load do
    #     # ...
    #   end
    #
    # @return [Array<Proc>]
    # @since 1.0.0
    # @api public
    def after_load(&block)
      @_after_load ||= []
      @_after_load << block if block_given?
      @_after_load
    end

    ####
    # @example
    #  Fenix.load_paths << Dir[Fenix.root('app', 'controllers', '**')]
    #
    # @return [Array<String>]
    # @since 1.0.0
    # @api public
    def load_paths
      @_load_paths ||= ['app/controllers/**', 'app/models/**', 'lib/**'].map do |directory|
        Dir[Fenix.root(directory)]
      end.flatten
    end
  end # self
end # Fenix