require_relative 'exceptions'
require_relative 'rendering'
require_relative 'settings'
require_relative 'response'
require_relative 'request'

module Fenix
  class Application
    attr_reader :env
    attr_reader :request
    attr_reader :response

    include Rendering

    class << self
      # @private
      def new(*arguments, &block)
        instance = super(*arguments, &block)
        stack.run(instance)
        stack
      end

      %w(GET HEAD POST PUT PATCH DELETE OPTIONS).each do |verb|
        define_method(verb.downcase) do |path, **conditions, &block|
          routes[verb] << compile_route(path, conditions, &block)
        end
      end

      # @private
      private def compile_route(path, conditions, &block)
        {
          conditions: conditions,
          object: Mustermann.new(path),
          block: block
        }
      end

      ####
      # @return [Hash]
      # @since 1.0.0
      # @api public
      def routes
        @_routes ||= Hash.new { |hash, key| hash[key] = [] }
      end

      ####
      # @return [Rack::Builder]
      # @see https://github.com/rack/rack/blob/master/lib/rack/builder.rb
      # @since 1.0.0
      # @api public
      def stack
        @_stack ||= Rack::Builder.new
      end

      ####
      # @param [options] Array<Symbol>
      #
      # @example
      #   enable :cache
      #
      # @since 1.0.0
      # @api public
      def enable(*options)
        options.each { |option| settings[option] = true }
      end

      ####
      # @param [options] Array<Symbol>
      #
      # @example
      #   disable :cache
      #
      # @since 1.0.0
      # @api public
      def disable(*options)
        options.each { |option| settings[option] = false }
      end

      ####
      # @return [Array]
      # @since 1.0.0
      # @api public
      def extensions
        @_extensions ||= []
      end

      ###
      # @param [extensions] Array
      #
      # @example
      #   register Fenix::Assets
      #   register Fenix::Cookies
      #
      # @since 1.0.0
      # @api public
      def register(*extensions)
        self.extensions += extensions
        extensions.each do |extension|
          extension.registered(self) if extension.respond_to?(:registered)
          extend extension
        end
      end

      ####
      # @return [Fenix::Settings]
      # @see Fenix::Settings
      # @since 1.0.0
      # @api public
      def settings
        @_settings ||= Fenix::Settings.new
      end
    end # self

    ####
    # @return [Fenix::Settings]
    # @see Fenix::Settings
    # @since 1.0.0
    # @api public
    def settings
      self.class.settings
    end

    # @return [Hash]
    # @since 1.0.0
    # @api public
    def session
      env['rack.session']
    end

    # @return [Symbol]
    # @since 1.0.0
    # @api public
    def locale
      I18n.locale
    end

    # @return [Hash]
    # @since 1.0.0
    # @api public
    def params
      request.params
    end

    # @return [Hash]
    # @since 1.0.0
    # @api public
    def cookies
      request.cookies
    end

    # @since 1.0.0
    # @api public
    def logger
      request.logger
    end

    # @since 1.0.0
    # @api public
    def redirect(uri, **notifications)
      if env['HTTP_VERSION'] == 'HTTP/1.1' and env['REQUEST_METHOD'] != 'GET'
        status = 303
      else
        status = 302
      end
      
      response['Location'] = uri
      halt(status)
    end

    # @since 1.0.0
    # @api public
    def halt(status = 500)
      response.status = status
      throw :halt
    end

    # @private
    def call(env)
      @env = env
      @request = Fenix::Request.new(env)
      @response = Fenix::Response.new

      dispatch
      @response.finish
    end

    # @private
    private def dispatch
      catch :halt do
        if route = find_route
          response.write(instance_eval(&route[:block]))
        else
          raise Fenix::NotFound
        end
      end
    end

    # @private
    private def find_route
      self.class.routes[request.request_method].detect do |route|
        if match = route[:object].match(request.path_info)
          match.names.each_with_index do |index, name|
            request[name] = match.captures[index]
          end
        end
      end
    end
  end # Application
end # Fenix